{% include "header.html" %}
<nav aria-label="breadcrumb">
    <ol class="breadcrumb bg-light">
        <li class="breadcrumb-item active" aria-current="page">Home</li>
        <li class="ms-auto json-link"><a target="_blank" href="./?f=json">JSON</a></li>
    </ol>
</nav>
<h1>MapServer Homepage</h1>
<p>
    This page lists all available Mapfiles hosted by MapServer.
</p>
<h2>Links</h2>
<div class="table-responsive">
    <table class="table" style="width:100%;">
        <thead class="thead-light">
            <tr>
                <th>#</th>
                <th>Key</th>
                <th>Name</th>
                <th>Layer Count</th>
                <th>HTML Documentation</th>
                <th>JSON</th>
            </tr>
        </thead>
        <tbody>
            {% for link in response.linkset -%}
            <tr>
                <td>{{ loop.index1 }}</td>
                <td>{{ link.key }}</td>
                <td>{{ link.title }}</td>
                <td>{{ link.layer-count }}</td>
                <td><a href="{{ link.service-doc.0.href }}">{{ link.service-doc.0.title }}</a></td>
                <td><a href="{{ link.service-desc.0.href }}">{{ link.service-desc.0.title }}</a></td>
            </tr>
            {% endfor %}
        </tbody>
    </table>
</div>
{% include "footer.html" -%}