{% if existsIn(template.tags, "debug") %}
  {% if template.tags.debug == "true" %}
<div class="card card-body bg-light" style="margin-top:20px;margin-bottom:20px;">
  <h4>Template Data</h4>
  <p>
    API Root: {{ template.api_root }}<br>
    Path: {{ template.path }}<br>
    Params: {{ template.params }}<br>
    Extra Params: {{ {{ template.extra_params }}<br>
    Title: {{ template.title }}<br>
    Tags: {{ template.tags }}<br>
  </p>
</div>
  {% endif %}
{% endif %}
